'********************
'***** EXWAVACQ ***** EXample of WAVeform ACQuisitions.
'********************
'***
'*** This example program reads waveform acquisitions and calculates the
'*** minimum, maximum, and delta values. Also a check is performed that
'*** the acquisition is ended within the specified time (pass/fail).
'*** Vmax, Vmin, Delta V, Delta T, Acq.Time, Pass/Fail are printed in a loop.
'*** After program start, the following is requested:
'*** - To select MEASUre -MEAS1-volt-max + -MEAS2-volt-min.
'*** - To enter the maximum acquisition time allowed in seconds.
'*** - To enter the index for the COM port to which the instrument is
'***   connected, e.g., 2 for COM2.
'*** - To enter the baudrate of the acquisitions, e.g., 9600.
'*** Note: The program automatically searches for the actual baudrate of
'***       the instrument, and then sets the baudrate specified.
'***
DECLARE SUB Waitforkey ()
DECLARE SUB Getbaudrate (Port%, baudrate&)
DECLARE SUB Send (com$)
DECLARE SUB ClearPort ()
DECLARE SUB WaitPort (MaxTime!)
DECLARE SUB WaitPc (Delay!)
 
DEFINT A-Z
DIM Wave#(512)                          ' Waveform Array
DIM Sample(1024)
Counter = 1                             ' Loop counter

CLS
PRINT
PRINT "          ͻ"
PRINT "           Put CombiScope instrument in 'Single Shot' Mode (STOP)."
PRINT "                                                                  "
PRINT "           Select MEASURE >>>MEAS1-volt-max + >>>MEAS2-volt-min.  "
PRINT "          ͼ"
PRINT

Waitforkey
PRINT

Read.Max.Acq.Time:         ' Reads maximum Acquisition Time in seconds
			   '  prints 'Pass' and > prints 'Fail'
  INPUT "Enter maximum acquisition time in seconds"; acq.time$
  AcqTime! = VAL(acq.time$)

Read.Com:
  PRINT
  INPUT "Enter index for the COM port, e.g.: 2 for COM2"; com.nr$
  com.nr% = VAL(com.nr$)
  IF com.nr% < 1 OR com.nr% > 4 GOTO Read.Com

Read.Baudrate:
  PRINT
  INPUT "Enter baudrate: 75, 150, 300, 600, 1200, 2400, 9600, 19200"; baud.rate$
  baud.rate& = VAL(baud.rate$)
  baud& = 75                              ' Starts speed at 75 bits per second
  WHILE baud.rate& <> baud&
    baud& = baud& * 2                     ' Doubles bit speed per second
    IF baud& > 19200 GOTO Read.Baudrate
  WEND

CALL Getbaudrate(com.nr%, baud.rate&)     ' Opens communications

REM ***
REM Set screen color.
REM ***
COLOR 7, 1
CLS

REM ***
REM Values returned by INKEY$.
REM ***
Quit$ = CHR$(0) + CHR$(45)              ' When ALT+x is pressed.
EnterKey$ = CHR$(13)                    ' When Enter is pressed.

REM ***
REM Set program name in header.
REM ***
COLOR 15, 1                             ' Sets screen color.
LOCATE 1, 3: PRINT "CombiScope Min/Max waveform acquisitions (max."; AcqTime!; "seconds)"
LOCATE 1, 67: PRINT "Version 1.0"
COLOR 7, 1                              ' Sets screen color.
LOCATE 2, 1
PRINT STRING$(80, "");
PRINT "  #       Vmax          Vmin         Delta V      Delta T     Acq.Time   P/F"
LOCATE 4, 1
PRINT STRING$(80, "");

REM ***
REM Set up prompt on bottom line of screen and turn cursor on.
REM ***
LOCATE 24, 1, 1
PRINT STRING$(80, "");
LOCATE 25, 1
PRINT TAB(30); "Press ALT+x to exit";

VIEW PRINT 5 TO 23                      ' Prints between lines 5 & 23.

REM ***
REM Main communication loop.
REM ***
DO
    KeyInput$ = INKEY$                  ' Checks the keyboard.
  
    IF KeyInput$ = Quit$ THEN           ' Exits the loop if the user
      EXIT DO                           ' pressed ALT+q.
    END IF
     
    T1! = TIMER                         ' Gets start time acquisition
   
    CALL ClearPort
   
    CALL Send("AT")                     ' Sends Arm Trigger
   
    CALL WaitPc(.2)                     ' Waits 0.2 seconds

    REM ***
    REM Get Waveform administration data.
    REM ***
    CALL Send("QW,001")                 ' Queries Waveform CH1
    CALL WaitPort(1)
    INPUT #1, Name$
    INPUT #1, Y.Unit$
    INPUT #1, X.Unit$
    INPUT #1, Y.Zero$
    INPUT #1, X.Zero$
    INPUT #1, Y.Resolution$
    INPUT #1, X.Resolution$
    INPUT #1, Y.Range&
    INPUT #1, Wave.Date$
    INPUT #1, Wave.Time$
    INPUT #1, Delta.T.Correction$
    INPUT #1, Min.Max.Trace$
    INPUT #1, Res1$
    INPUT #1, Res2$
    INPUT #1, Res3$
    INPUT #1, Res4$
    INPUT #1, Count
    FOR i = 1 TO Count * 2
       Sample(i) = ASC(INPUT$(1, #1))
    NEXT i
    CheckSum = ASC(INPUT$(1, #1))
  
    T2! = TIMER                         ' Gets end time acquisition

'   PRINT Name$
'   PRINT Y.Unit$, X.Unit$
'   PRINT Y.Zero$, X.Zero$
'   PRINT Y.Resolution$, X.Resolution$
'   PRINT Y.Range&
'   PRINT Wave.Date$, Wave.Time$
'   PRINT Delta.T.Correction$, Min.Max.Trace$
'   PRINT Res1$, Res2$, Res3$, Res4$
'   PRINT Count
'   FOR i = 1 TO Count * 2
'     PRINT Sample(i);
'   NEXT i
'   PRINT
'   PRINT Sum, CheckSum

    PRINT #1, "QM";                   ' Sends Query Measurement
    PRINT #1, 1;                      ' N = 1
    CALL Send("V")                    ' V = only value
    INPUT #1, Max$                    ' Inputs Maximum
    PRINT #1, "QM";                   ' Sends Query Measurement
    PRINT #1, 2;                      ' N = 2
    CALL Send("V")                    ' V = only value
    INPUT #1, Min$                    ' Inputs Minimum
    
    Delta# = VAL(Max$) - VAL(Min$)    ' Calculates Delta Peak-Peak

    REM ***
    REM Convert wave string into a wave array and get the index value
    REM with the Maximum and the index with the Minimum voltage
    REM to calculate delta time.
    REM ***
    Quit = 0
    FOR i = 1 TO Count
      REM ***
      REM Convert the string value into a double precision value.
      REM ***
      j = i * 2 - 1
      IF Sample(j) < 128 THEN
	trace& = Sample(j) * 256 + Sample(j + 1)
	ELSE trace& = (Sample(j) - 256) * 256 + Sample(j + 1)
      END IF
      Wave#(i) = VAL(Y.Resolution$) * trace&
      IF VAL(Max$) = Wave#(i) THEN
	Maxindex = i
      END IF
      IF VAL(Min$) = Wave#(i) AND Quit = 0 THEN
	Minindex = i
	Quit = 1
      END IF
    NEXT i
     
    REM ***
    REM Calculate the real delta time.
    REM ***
    DeltaTime# = (Maxindex - Minindex) * VAL(X.Resolution$)
    
    PRINT USING "####"; Counter;           ' Prints Counter
    PRINT SPACE$(3);
    PRINT USING "####.####"; VAL(Max$);    ' Prints Maximum Peak
    PRINT SPACE$(5);
    PRINT USING "####.####"; VAL(Min$);    ' Prints Minimum Peak
    PRINT SPACE$(4);
    PRINT USING "####.####"; Delta#;       ' Prints Delta Peak-Peak
    PRINT SPACE$(4);
    PRINT USING "####.######"; DeltaTime#; ' Prints Delta Time
    PRINT SPACE$(2);
    PRINT USING "###.###"; T2! - T1!;      ' Prints Acquisition time
    PRINT SPACE$(6);

    REM ***
    REM Check The Maximum Time.
    REM ***
    IF T2! - T1! > AcqTime! THEN
      PRINT "Fail"
    ELSE
      PRINT "Pass"
    END IF

    Counter = Counter + 1
LOOP

CLOSE                                           ' Ends communications.
END

SUB ClearPort
   
  WHILE LOC(1) > 0
    Dummy$ = INPUT$(1, #1)
  WEND

END SUB

SUB Getbaudrate (Port, baudrate&)

  REM ***
  REM Open communications, set Baudrate, no parity, 8-bit data,
  REM 1 stop bit, 2048-byte input buffer.
  REM ***

  CLS

  baud& = baudrate&
  FOR i = 0 TO 10
    PRINT TAB(25); "Checking Baudrate at:"; baud&
    Device$ = "COM" + LTRIM$(STR$(Port)) + ":" + LTRIM$(STR$(baud&)) + ",N,8,1,RB2048"
    OPEN Device$ FOR RANDOM AS #1 LEN = 2048
    PRINT #1, CHR$(27);                         ' Esc character
    WaitPc (.5)
    PRINT #1, CHR$(13);                         ' CR character
    WaitPc (.5)
    IF LOC(1) > 1 THEN
      Ack$ = INPUT$(1, #1)
      Term$ = INPUT$(1, #1)
    END IF
   
    IF Ack$ = "0" THEN
      CALL Send("PC," + LTRIM$(STR$(baudrate&)) + ",N,8,1,XONXOFF")
      CLOSE
      baud& = baudrate&
      Device$ = "COM" + LTRIM$(STR$(Port)) + ":" + LTRIM$(STR$(baud&)) + ",N,8,1,RB2048"
      OPEN Device$ FOR RANDOM AS #1 LEN = 2048
      EXIT FOR
    END IF

    CLOSE
    IF baud& = 19200 THEN
      baud& = 75
    ELSE baud& = baud& * 2
    END IF

  NEXT i
 
END SUB

DEFSNG A-Z
SUB Send (com$)
 
   Retry = 3                                    ' Number of Retries

   FOR i = 1 TO Retry
   
     PRINT #1, com$                             ' Sends command

     CALL WaitPort(.5)
     Ack$ = INPUT$(1, #1)                       ' Gets the acknowledge
    
     CALL WaitPort(.3)
     Zackterm$ = INPUT$(1, #1)                  ' Gets the termination
  
     IF Ack$ <> "0" THEN
       BEEP                                     ' Beeps and sends command again
     ELSE
       EXIT FOR
     END IF

   NEXT i

   IF i = Retry THEN
     PRINT "There were " + STR$(Retry) + " attempts in order to get an answer from " + com$
   END IF

END SUB

DEFINT A-Z
SUB Waitforkey
PRINT "Press any key to continue."
Ky$ = INKEY$
WHILE Ky$ = ""
  Ky$ = INKEY$
WEND
END SUB

SUB WaitPc (Delay!)

  Time1! = TIMER
  DO
    Time2! = TIMER
  LOOP WHILE Time2! - Time1! <= Delay!

END SUB

SUB WaitPort (MaxTime!)
   
   Time1! = TIMER
   DO
     Time2! = TIMER
   LOOP UNTIL LOC(1) > 0 OR Time2! - Time1! > MaxTime!

END SUB

