REM $INCLUDE: 'QBDECL.BAS'
'*****              *****
'***** EXAPPA12.BAS *****
'***** EXample program APPendix A.1.2 in the SCPI Programming Manual.
'*****              *****
DECLARE SUB errorcheck ()
DIM res AS STRING * 100           'Dimension response string
DIM cmd AS STRING                 'Declare command string
EndEOI% = 1                       'Termination Send on LineFeed & EOI
StopEOI% = 256                    'Termination Receive on EOI
' **************************************************
' ******** Program start ***************************
' **************************************************
CLS                               'Clears Output Screen
CALL SendIFC(0)                   'Clears the GPIB interface
CALL IBTMO(0, 13)                 'Timeout at 10 seconds
'
' Reset the instrument and clear the status data.
'
cmd$ = "*rst;*cls"
CALL Send(0, 8, cmd$, EndEOI%)
'
' Program AC input coupling + continuous trigger initiation.
'
cmd$ = "INPut:COUPling AC;:INITiate:CONTinuous ON"
CALL Send(0, 8, cmd$, EndEOI%)
'
' Program channel 1 as trigger source + trigger level zero.
'
cmd$ = "TRIGger:SOURce INTernal1;:TRIGger:LEVel 0.0"
CALL Send(0, 8, cmd$, EndEOI%)
'
' Program 100 s MTB + 200 mV attenuation.
' MTB = (sweep time/(acq.length-1))*50 = (1000/511)*50 = 100 s.
' Attenuation = peak-to-peak/number of divisions = 1600/8 = 200 mV.
'
cmd$ = "SENSe:SWEep:TIME 1E-3;:SENSe:VOLTage:RANGe:PTPeak 1.6"
CALL Send(0, 8, cmd$, EndEOI%)
CALL errorcheck
'
PRINT "Make an overshoot on the rising edge of the CAL signal."
PRINT ">>> Press any key when finished."
WHILE INKEY$ = "": WEND: PRINT
'
' Disable the initiation of the trigger system.
'
cmd$ = "INITiate:CONTinuous OFF"
CALL Send(0, 8, cmd$, EndEOI%)
'
' Read and print one peak-to-peak value.
'
cmd$ = "READ:PTPeak?"
CALL Send(0, 8, cmd$, EndEOI%)
CALL Receive(0, 8, res$, StopEOI%)     'Enters Peak_To_Peak value
PRINT "Peak_To_Peak value = ";
PRINT LEFT$(res$, INSTR(res$, CHR$(10)) - 1); " V"
PRINT
ptpeak = VAL(res$)                     'Conversion to numeric
'
' Fetch and print the risetime overshoot percentage.
'
cmd$ = "FETCh:RISE:OVERshoot?"
CALL Send(0, 8, cmd$, EndEOI%)
CALL Receive(0, 8, res$, StopEOI%)     'Enters risetime overshoot %
PRINT "Rise_Time_Overshoot perc. = ";
PRINT LEFT$(res$, INSTR(res$, CHR$(10)) - 1); " %"
PRINT
rtoshoot = VAL(res$)                   'Conversion to numeric
'
' Calculate and print the risetime overshoot value.
' Rise time overshoot value = (rtoshoot / 100) * ptpeak
'
rtoshoot = (rtoshoot / 100) * ptpeak
PRINT "Rise_Time_Overshoot value ="; rtoshoot; "V"
PRINT
CALL errorcheck
' **************************************************
' ******** Program end *****************************
' **************************************************
END

' **************************************************
' Subroutine reading one error from the error queue.
' **************************************************
SUB errorcheck
  er$ = SPACE$(1)
  WHILE LEFT$(er$, 1) <> "0"
  cmd$ = ":SYSTem:ERRor?"
  CALL Send(0, 8, cmd$, 1)                             'Sends error query
  er$ = SPACE$(60)
  CALL Receive(0, 8, er$, 256)                        'Reads error string
  PRINT "error = "; er$                            'Displays error string
  WEND
END SUB

