REM $INCLUDE: 'QBDECL.BAS'
'*****              *****
'***** EXAPPA32.BAS *****
'***** EXample program APPendix A.3.2 in the SCPI Programming Manual.
'*****              *****
'
'***** LEETESTE.BAS ***** Programs TEST1/2 by saving/restoring
'                         to/from disk using SYSTem:SETtings.
DECLARE SUB Errorcheck ()
DIM SHARED setupin AS STRING * 400
CLS
CALL SendIFC(0)
CALL IbTMO(0, 13)                        'Time out of 10 seconds
CALL Send(0, 8, "*CLS", 1)               'Clears old status data

SRQ.detected = 0
ON PEN GOSUB ServReq                     'Defines SRQ-routine

'*** Enable SRQ generation after ESR bit 1 "Operation Completed".
CALL Send(0, 8, "*ESE 1", 1)             'ESR-OPC bit = enabled

'*** Operation completed is reported in bit 5 (ESB) of STB after sending *OPC.
CALL Send(0, 8, "*SRE 32", 1)            'STB-ESB bit = enabled

'*** This results in the generation of SRQ after "Operation Completed".
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
'*** However, in most cases the mouse software may not be loaded or
'*** active in the computer. So, remove the "mouse load statement"
'*** from the CONFIG.SYS or AUTOEXEC.BAT file.
'<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

'*** Programs the actual setup by saving/reading instrument
'*** settings (SYSTem:SET) to/from hard disk.
'
INPUT "Press y + Enter to SAVE the current instrument setup to disk."; reply$
IF (reply$ = "Y") OR (reply$ = "y") THEN GOSUB Save.Setup
PRINT
'
INPUT "Press y + Enter to READ an instrument setup from disk."; reply$
IF (reply$ = "Y") OR (reply$ = "y") THEN GOSUB Enter.Setup
PRINT

CALL Send(0, 8, "INITiate:CONTinuous OFF", 1) 'Sets Auto trig off
CALL Send(0, 8, "ABORt", 1)                   'Aborts current meas.
PEN ON                                        'Enables SRQ-routine

Repeat.test1:
  CALL Send(0, 8, "INITiate;*OPC", 1)         'Single shot meas.
  timeout = 10                                'Time out of 10 seconds.
  WHILE SRQ.detected = 0
    'Do something else ?
    PRINT ".";
    SLEEP (1)                                 'Delays 1 second
    IF timeout = 0 THEN
       SRQ.detected = 1
       PRINT "Time out on SRQ !": PRINT
    END IF
    timeout = timeout - 1
  WEND
  SRQ.detected = 0
  CALL Send(0, 8, "DISPlay:WINDow:TEXT20:DATA?", 1) 'Requests for dT
  cursdata$ = "               "
  CALL Receive(0, 8, cursdata$, 256)        'Enters dT cursor readout
  PRINT "Cursor readout: "; cursdata$
  CALL Errorcheck
  PRINT "Press any key for next cursor-readout or Ctrl/Break to stop."
  WHILE INKEY$ = "": WEND
GOTO Repeat.test1

END
'
'*** Service Request routine to generate SRQ after "Operation Completed".
'
ServReq:
  PRINT "Service Request generated"
  CALL ReadStatusByte(0, 8, sbyte%) 'Serial polls status byte to reset SRQ
  CALL Send(0, 8, "*ESR?", 1)       'Queries ESR to clear OPC bit
  esr$ = "   "
  CALL Receive(0, 8, esr$, 256)     'Enters ESR status byte
  SRQ.detected = 1
RETURN
'
'*** Routine to enter an instrument setup from file + download it.
'
Enter.Setup:
  INPUT "Enter 'path/directory/file_name' of the setup file."; replyin$
  OPEN replyin$ FOR INPUT ACCESS READ AS #1
    LINE INPUT #1, len$
    length% = VAL(len$)
    PRINT "Instrument setup length ="; length%; "bytes"
    setupout$ = ""                      'Empty string
    WHILE NOT (EOF(1))
      B$ = "."
      B$ = INPUT$(1, #1)                'Input 1 byte
      setupout$ = setupout$ + B$        'Add to setup string
    WEND
    CLOSE
    OPEN "O", #1, "setupin.dat"         'Writes settings to SETUPIN.DAT
      PRINT #1, setupout$
    CLOSE
  CALL Send(0, 8, "SYSTem:SET ", 0)               'Sends SCPI header
  CALL Send(0, 8, LEFT$(setupout$, length%), 0)   'Sends settings buffer
  CALL Send(0, 8, "", 1)                          'Sends EOL termination
  CALL Errorcheck
RETURN
'
'*** Routine to upload the current instrument setup + save it to file.
'
Save.Setup:
  INPUT "Enter 'path/directory/file_name' of the setup file."; replyout$
  CALL Send(0, 8, "SYSTem:SET?", 1)       'Requests for settings
  CALL Receive(0, 8, setupin$, 256)       'Reads the settings
  length% = IBCNT%
  PRINT "Number of system settings bytes = "; IBCNT%
  OPEN "O", #1, replyout$        'Writes settings to file name in REPLYOUT$
    PRINT #1, length%
    PRINT #1, setupin$
  CLOSE
  CALL Errorcheck
RETURN

'
' **************************************************
' Subroutine reading one error from the error queue.
' **************************************************
SUB Errorcheck
  er$ = SPACE$(1)
  WHILE LEFT$(er$, 1) <> "0"
  CMD$ = ":SYSTem:ERRor?"
  CALL Send(0, 8, CMD$, 1)            'Sends error query
  er$ = SPACE$(60)
  CALL Receive(0, 8, er$, 256)        'Reads error string
  PRINT "error = "; er$               'Displays error string
  WEND
END SUB

