REM $INCLUDE: 'QBDECL.BAS'
'*****
'***** EXAPPA53.BAS
'***** EXample program APPendix A.5.3 in the SCPI Programming Manual.
'*****
'***** This program starts up the pass/fail test, monitors it, and
'***** reads and saves failing waveforms to disk.
'*****
DECLARE SUB Errorcheck ()
DIM res AS STRING * 100               'Dimension response string
DIM trace AS STRING * 5000            'Dimension trace buffer
DIM cmd AS STRING                     'Declare command string
DIM address%(2)
EndEOI% = 1                           'Termination Send on LineFeed & EOI
StopEOI% = 256                        'Termination Receive on EOI
' **************************************************
' ******** Program start ***************************
' **************************************************
CLS                                   'Clears Output Screen
CALL SendIFC(0)                       'Clears the GPIB interface
CALL IBTMO(0, 13)                     'Timeout at 10 seconds
'***
'*** Enable Pass/Fail status (10) bit to be reported in OPERation register.
'***
cmd$ = "STATus:OPERation:ENABle 1024"
CALL Send(0, 8, cmd$, EndEOI%)
'***
'*** Enable OPERation status event to be reported in bit 7 (OPER) of STB.
'***
CALL Send(0, 8, "*SRE 128", 1)           'STB-ESB bit = enabled
ON PEN GOSUB ServReq                     'Defines SRQ-routine
PEN ON                                   'Enables SRQ-routine
'***
'*** Now SRQ generation after "OPERation event" is enabled.
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
'*** However, in most cases the mouse software may not be loaded or
'*** active in the computer. So, remove the "mouse load statement"
'*** from the CONFIG.SYS or AUTOEXEC.BAT file first.
'<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

OPEN "O", #1, "FAILTRAC.DAT"            'Opens the file FAILTRAC.DAT
'***
'*** Switch PASS/FAIL testing on. Condition = PASS/FAIL must be off !
'***
cmd$ = "DISPlay:MENU MEASure;:SYSTem:KEY 6;KEY 5;:DISPlay:MENU:STATe OFF"
'*** SYSTem:KEY 6 selects PASS/FAIL.
'*** SYSTem:KEY 5 selects run (turns pass/fail checking on).
CALL Send(0, 8, cmd$, EndEOI%)
PRINT "Check that pass/fail checking is started (PASS or FAIL displayed)."
PRINT "If not >>> press Ctrl/Break and restart this program."
SRQ.detected = 0
'***
'*** Start of the FAIL check loop (5 times).
'***
FOR trace.count = 1 TO 5
  WHILE SRQ.detected = 0
    PRINT ".";
    '>>> do something else ! <<<
    SLEEP (1)
  WEND
  SRQ.detected = 0
  '
  '*** A FAIL situation has occurred; turn Pass/Fail checking off.
  '*** :SYSTem:KEY 6   = PASS/FAIL
  '*** :SYSTem:KEY 5   = stop (turns pass/fail checking off)
  cmd$ = "DISPlay:MENU MEASure;:SYSTem:KEY 6;KEY 5"
  CALL Send(0, 8, cmd$, EndEOI%)
  cmd$ = "SYSTem:BEEPer"
  CALL Send(0, 8, cmd$, EndEOI%)  'Generates a beep of 1 second
  '***
  '*** Read fail trace M3_1 to trace$ + write to file "failtrac.dat"
  '***
  cmd$ = "TRACe? M3_1"
  CALL Send(0, 8, cmd$, EndEOI%)          'Requests for trace M3_1
  CALL Receive(0, 8, trace$, StopEOI%)    'Reads the trace buffer
  PRINT "Length of the trace buffer ="; IBCNT%
  PRINT #1, LEFT$(trace$, IBCNT%)         'Writes trace to file
  PRINT "Fail trace M2_1 saved to file 'failtrac.dat' as trace"; trace.count
  cmd$ = "SYSTem:KEY 5;:DISPlay:MENU:STATe OFF"
  '*** SYSTem:KEY 5 selects run (turns pass/fail checking on).
  CALL Send(0, 8, cmd$, EndEOI%)
  CALL Errorcheck
NEXT trace.count
CLOSE                                   'Closes the file FAILTRAC.DAT
cmd$ = "DISPlay:MENU MEASure;:SYSTem:KEY 6;KEY 5;:DISPlay:MENU:STATe OFF"
'*** SYSTem:KEY 6 selects PASS/FAIL.
'*** SYSTem:KEY 5 selects stop (turns pass/fail checking off).
CALL Send(0, 8, cmd$, EndEOI%)
' **************************************************
' ******** Program end *****************************
' **************************************************
END
'
'*** Service Request routine to generate SRQ after "Operation Completed".
'
ServReq:
  PRINT "Service Request generated"
  '***
  '*** Serial poll status byte to reset SRQ.
  '***
  CALL ReadStatusByte(0, 8, sbyte%)
  '***
  '*** Read the OPERation event register to clear the FAIL event.
  '***
  CALL Send(0, 8, "STATus:OPERation:EVENt?", EndEOI%)
  CALL Receive(0, 8, res$, StopEOI%)
  SRQ.detected = 1
RETURN

' **************************************************
' Subroutine reading one error from the error queue.
' **************************************************
SUB Errorcheck
  er$ = SPACE$(1)
  WHILE LEFT$(er$, 1) <> "0"
  cmd$ = ":SYSTem:ERRor?"
  CALL Send(0, 8, cmd$, 1)                             'Sends error query
  er$ = SPACE$(60)
  CALL Receive(0, 8, er$, 256)                        'Reads error string
  PRINT "error = "; er$                            'Displays error string
  WEND
END SUB

