'*****
'***** EXAPPB53.BAS (for TestTeam Plus and LabWindows).
'***** EXample program APPendix A.5.3 in the SCPI Programming Manual.
'*****
'***** This program starts the Pass/fail Test and monitors it. When a fail
'***** occurs, the failing waveform is read from M3_1 and stored in the
'***** file FAILWAVE.DAT on the hard disk.
'***** Note: Program - Portability Checking must be switched off.
'*****
DEFINT A-Z
DIM SHARED res AS STRING * 100               'Dimension response string
DIM SHARED fail.wave AS STRING * 5000        'Dimension trace buffer
DIM SHARED cmd AS STRING                     'Declare command string
DIM address%(2)
COMMON SHARED /trace.count/ trace.count
COMMON SHARED /SRQ.detected/ SRQ.detected
DECLARE SUB Serv.Req()

' **************************************************
' ******** Program start ***************************
' **************************************************
CLS                                   'Clears Output Screen
CALL SendIFC(0)                       'Clears the GPIB interface
CALL IBTMO(0, 13)                     'Sets timeout at 10 seconds
CALL Send(0, 8, "*CLS", 4, 1)         'Clears status data

'***
'*** Enable Pass/Fail status (10) bit to be reported in OPERation register.
'***
cmd$ = "STATus:OPERation:ENABle 1024"
length& = StringLength (cmd$)
CALL Send(0, 8, cmd$, length&, 1)
'***
'*** Enable OPERation status event to be reported in bit 7 (OPER) of STB.
'***
CALL Send(0, 8, "*SRE 128", 8, 1)        'STB-ESB bit = enabled
ON PEN GOSUB SRQ.statement               'Defines SRQ-routine
PEN ON                                   'Enables SRQ-routine
'***
'*** Now SRQ generation after "OPERation event" is enabled.
'>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
'*** However, in most cases the mouse software may not be loaded or
'*** active in the computer. So, remove the "mouse load statement"
'*** from the CONFIG.SYS or AUTOEXEC.BAT file first.
'*** Also, "Automatic Serial Polling" must be disabled using the
'*** IBCONF.EXE utility.
'<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<

'***
'*** Open the file FAILWAVE.DAT for binary output.
'***
file.handle = OpenFile("FAILWAVE.DAT", 2, 0, 0)
IF file.handle < 0 THEN
  PRINT "Error opening file."
  STOP
END IF

'***
'*** Switch PASS/FAIL testing on. Condition = PASS/FAIL must be off !
'***
cmd$ = "DISPlay:MENU MEASure;:SYSTem:KEY 6;KEY 5;:DISPlay:MENU:STATe OFF"
'*** SYSTem:KEY 6 selects PASS/FAIL.
'*** SYSTem:KEY 5 selects run (turns pass/fail checking on).
length& = StringLength (cmd$)
CALL Send(0, 8, cmd$, length&, 1)
PRINT "Check that pass/fail checking is started (PASS or FAIL displayed)."
PRINT "If not >>> press Ctrl/Break and restart this program."
SRQ.detected = 0

'***
'*** Start of the FAIL check loop (5 times).
'***
FOR trace.count = 1 TO 5
  WHILE SRQ.detected = 0
    dummy = 0
    '>>> do something else ! <<<
  WEND
  SRQ.detected = 0

  '*** A FAIL situation has occurred; turn Pass/Fail checking off.
  '*** SYSTem:KEY 6   = PASS/FAIL
  '*** SYSTem:KEY 5   = stop (turns pass/fail checking off)
  cmd$ = "DISPlay:MENU MEASure;:SYSTem:KEY 6;KEY 5"
  length& = StringLength (cmd$)
  CALL Send(0, 8, cmd$, length&, 1)
  cmd$ = "SYSTem:BEEPer"
  length& = StringLength (cmd$)
  CALL Send(0, 8, cmd$, length&, 1)           'Generates a beep of 1 second

  '***
  '*** Read fail trace M3_1 to fail.wave$ + write to file "failwave.dat"
  '***
  CALL Send(0, 8, "TRACe? M3_1", 11, 2)
  CALL Receive(0, 8, fail.wave$, 5000, 256)
  lengthe = IBCNT%
  PRINT "Number of trace bytes of failing wave ="; lengthe
  CALL Send(0, 8, "SYSTem:ERRor?", 13, 1)
  CALL Receive(0, 8, res$, 80, 256)
  PRINT res$
  n = CompareBytes(res$, 0, "0", 0, 1, 0)
  IF n <> 0 THEN
    PRINT "Program stopped."
    STOP
  END IF

  '***
  '*** Save the read failing wave to file FAILWAVE.DAT.
  '***
  n = FmtFile(file.handle, "%s<%i", lengthe)
  IF n <> 1 THEN
    PRINT "Error writing fail.wave.length to file."
    STOP
  END IF
  n = WriteFile(file.handle, CHR$(10), 1)
  n = FmtFile(file.handle, "%s<%s[w1033t-]", fail.wave$)
  IF n <> 1 THEN
    PRINT "Error writing failing waveform to file."
    STOP
  END IF
  cmd$ = "SYSTem:KEY 5;:DISPlay:MENU:STATe OFF"
  '*** SYSTem:KEY 5 selects run (turns pass/fail checking on).
  length& = StringLength (cmd$)
  CALL Send(0, 8, cmd$, length&, 1)

NEXT trace.count                        'Repeat 5 times

n = CloseFile(file.handle)              'Closes the file FAILWAVE.DAT

cmd$ = "DISPlay:MENU MEASure;:SYSTem:KEY 6;KEY 5;:DISPlay:MENU:STATe OFF"
'*** SYSTem:KEY 6 selects PASS/FAIL.
'*** SYSTem:KEY 5 selects stop (turns pass/fail checking off).
length& = StringLength (cmd$)
CALL Send(0, 8, cmd$, length&, 1)
CALL Send(0, 8, "*SRE 0", 6, 1)          'STB-ESB bit = disabled

PRINT "Pass/fail testing ended."
' **************************************************
' ******** Program end *****************************
' **************************************************

'***
'*** Service Request routine handler".
'***
SUB Serv.Req
  PRINT "Service Request"; trace.count; "generated."
  '***
  '*** Serial poll status byte to reset SRQ.
  '***
  CALL ReadStatusByte(0, 8, sbyte)
  '***
  '*** Read the OPERation event register to clear the FAIL event.
  '***
  CALL Send(0, 8, "STATus:OPERation:EVENt?", 23, 1)
  CALL Receive(0, 8, res$, 5, 256)
  SRQ.detected = 1
END SUB

END

'***
'*** Service Request statement called after SRQ at fail trace event".
'***
SRQ.statement:
  CALL Serv.Req                 'Execute service request handling
RETURN
