REM $INCLUDE: 'QBDECL.BAS'
'************************
'***** EXCNVTRC.BAS ***** Reading, converting, and printing a trace of 512
'************************ samples of 8 bits (1 byte) or 16 bits (2 bytes)
'************************ from channel 1.
DECLARE SUB errorcheck ()
DIM trace%(512)                           'Array of 512 integers
DIM sample(512)                           'Array of 512 reals
DIM response AS STRING * 1033             'Trace data response string
DIM peaktop AS STRING * 10                'Peak-to-peak response string
DIM offset AS STRING * 10                 'Offset response string

CLS
CALL SendIFC(0)
CALL IBTMO(0, 13)                         'Time out of 10 seconds

CALL Send(0, 8, "*RST", 1)                'Resets the instrument
CALL Send(0, 8, "CONFigure:AC (@1)", 1)   'Configures for AC-RMS
CALL Send(0, 8, "INITiate", 1)            'Initiates single acquisition
CALL Send(0, 8, "*WAI;TRACe? CH1", 1)     'Requests channel 1 trace
CALL Receive(0, 8, response$, 256)        'Reads channel 1 trace
PRINT "Number of read bytes ="; IBCNT%    'IBCNT% = length of trace buffer

nr.of.digits = VAL(MID$(response$, 2, 1))
nr.of.bytes = VAL(MID$(response$, 3, nr.of.digits)) - 2
sample.length = ASC(MID$(response$, 3 + nr.of.digits, 1))
nr.of.samples = nr.of.bytes / (sample.length / 8)
PRINT "Number of trace samples  ="; nr.of.samples

CALL Send(0, 8, "SENSe:VOLTage:RANGe:PTPeak?", 1)  'Queries peak-to-peak
CALL Receive(0, 8, peaktop$, 256)                  'Reads peak-to-peak
ptpeak = VAL(LEFT$(peaktop$, IBCNT%))
PRINT "peak-to-peak = "; ptpeak
CALL Send(0, 8, "SENSe:VOLTage:RANGe:OFFSet?", 1)  'Queries offset
CALL Receive(0, 8, offset$, 256)                   'Reads offset
offs = VAL(LEFT$(offset$, IBCNT%))
PRINT "Offset volt. = "; offs
CALL errorcheck

IF sample.length = 1 THEN
   FOR i = 1 TO nr.of.samples                   '1-byte samples
      trace%(i) = ASC(MID$(response$, i + 3 + nr.of.digits, 1))
      IF trace%(i) > 127 THEN
	 trace%(i) = trace%(i) - 256
      END IF
      sample(i) = trace%(i) / 200 * ptpeak - offs
      PRINT sample(i);
      'PRINT trace%(i);
   NEXT i
ELSE
   FOR i = 1 TO nr.of.samples                   '2-byte samples
      J = 2 * i + 2 + nr.of.digits              'Pointer to next sample
      byte1 = ASC(MID$(response$, J, 1))        'M.S.B.
      byte2 = ASC(MID$(response$, J + 1, 1))    'L.S.B.
      IF byte1 < 128 THEN
	 trace%(i) = byte1 * 256 + byte2
      ELSE trace%(i) = (byte1 - 256) * 256 + byte2
      END IF
      sample(i) = trace%(i) / 51200 * ptpeak - offs
      PRINT sample(i);
      'PRINT trace%(i);
   NEXT i
END IF
PRINT : CALL errorcheck
END

'
' **************************************************
' Subroutine reading one error from the error queue.
' **************************************************
SUB errorcheck
  er$ = SPACE$(1)
  WHILE LEFT$(er$, 1) <> "0"
  CMD$ = ":SYSTem:ERRor?"
  CALL Send(0, 8, CMD$, 1)            'Sends error query
  er$ = SPACE$(60)
  CALL Receive(0, 8, er$, 256)        'Reads error string
  PRINT "error = "; er$               'Displays error string
  WEND
END SUB

