REM $INCLUDE: 'QBDECL.BAS'
'************************
'***** EXFFTTRC.BAS ***** Reading, converting, and printing of ABSolute or
'************************ RELative FFT cursor values from a trace of 512
'************************ samples of 8 bits (1 byte) or 16 bits (2 bytes)
'************************ from channel 1 via MATH1. Pre-conditions:
'************************ > MATH1 must be "on" and "fft" selected.
'************************ > CURSORS must be "on" and "m1.1" selected.
'************************ > MATH-PARAM-READOUT --> "abs" or "rel".
'************************ > CURSORS-READOUT + cursor-TRACK-control.
DECLARE SUB errorcheck ()
DIM trace&(512)                         'Array of 512 integers
DIM amdb(512)                           'Array of 512 "dB" amplitudes
DIM amdbm50(512)                        'Array of 512 "dBm 50" amplitudes
DIM amdbm600(512)                       'Array of 512 "dBm 100" amplitudes
DIM amdbmuv(512)                        'Array of 512 "dBV" amplitudes
DIM amvrms(512)                         'Array of 512 "Vrms" amplitudes
DIM freq(512)                           'Array of 512 frequencies
DIM response AS STRING * 1033
DIM sweeptime AS STRING * 15
DIM peak.to.peak AS STRING * 15
DIM acqlength AS STRING * 10
DIM ref(5, 11)                          'Reference conversion table
'*** Attenuation reference values.
ref(1, 1) = 5                           ' 5   V/div
ref(1, 2) = 2                           ' 2   V/div
ref(1, 3) = 1                           ' 1   V/div
ref(1, 4) = .5                          ' 0.5 V/div
ref(1, 5) = .2                          ' 0.2 V/div
ref(1, 6) = .1                          ' 0.1 V/div
ref(1, 7) = .05                         '50  mV/div
ref(1, 8) = .02                         '20  mV/div
ref(1, 9) = .01                         '10  mV/div
ref(1, 10) = .005                       ' 5  mV/div
ref(1, 11) = .002                       ' 2  mV/div
'*** Vrms factor reference values.
ref(2, 1) = 11.18034                    'Vrms -  5   V/div
ref(2, 2) = 4.4721359#                  'Vrms -  2   V/div
ref(2, 3) = 2.236068                    'Vrms -  1   V/div
ref(2, 4) = 1.118034                    'Vrms -  0.5 V/div
ref(2, 5) = .4472136#                   'Vrms -  0.2 V/div
ref(2, 6) = .2236068#                   'Vrms -  0.1 V/div
ref(2, 7) = .1118034#                   'Vrms - 50  mV/div
ref(2, 8) = .0447214#                   'Vrms - 20  mV/div
ref(2, 9) = .0223607#                   'Vrms - 10  mV/div
ref(2, 10) = .0111803#                  'Vrms -  5  mV/div
ref(2, 11) = .0044721#                  'Vrms -  2  mV/div
'*** dBm - 50 offset reference values.
ref(3, 1) = 33.9794                     'dBm(50) -  5   V/div
ref(3, 2) = 26.0206                     'dBm(50) -  2   V/div
ref(3, 3) = 20!                         'dBm(50) -  1   V/div
ref(3, 4) = 13.9794                     'dBm(50) -  0.5 V/div
ref(3, 5) = 6.0206                      'dBm(50) -  0.2 V/div
ref(3, 6) = 0!                          'dBm(50) -  0.1 V/div
ref(3, 7) = -6.0206                     'dBm(50) - 50  mV/div
ref(3, 8) = -13.979392#                 'dBm(50) - 20  mV/div
ref(3, 9) = -20.0                       'dBm(50) - 10  mV/div
ref(3, 10) = -26.020632#                'dBm(50) -  5  mV/div
ref(3, 11) = -33.97947                  'dBm(50) -  2  mV/div
'*** dBm - 600 offset reference values.
ref(4, 1) = 23.187588#                  'dBm(600) -  5   V/div
ref(4, 2) = 15.228787#                  'dBm(600) -  2   V/div
ref(4, 3) = 9.208187199999999#          'dBm(600) -  1   V/div
ref(4, 4) = 3.1875874#                  'dBm(600) -  0.5 V/div
ref(4, 5) = -4.771213                   'dBm(600) -  0.2 V/div
ref(4, 6) = -10.791813#                 'dBm(600) -  0.1 V/div
ref(4, 7) = -16.812413#                 'dBm(600) - 50  mV/div
ref(4, 8) = -24.771206#                 'dBm(600) - 20  mV/div
ref(4, 9) = -30.791813#                 'dBm(600) - 10  mV/div
ref(4, 10) = -36.812444#                'dBm(600) -  5  mV/div
ref(4, 11) = -44.771282#                'dBm(600) -  2  mV/div
'*** dBV offset reference values.
ref(5, 1) = 140.9691                    'dBV -  5   V/div
ref(5, 2) = 133.0103                    'dBV -  2   V/div
ref(5, 3) = 126.9897                    'dBV -  1   V/div
ref(5, 4) = 120.9691                    'dBV -  0.5 V/div
ref(5, 5) = 113.0103                    'dBV -  0.2 V/div
ref(5, 6) = 106.9897                    'dBV -  0.1 V/div
ref(5, 7) = 100.9691                    'dBV - 50  mV/div
ref(5, 8) = 93.01030799999999#          'dBV - 20  mV/div
ref(5, 9) = 86.98970799999999#          'dBV - 10  mV/div
ref(5, 10) = 80.96907                   'dBV -  5  mV/div
ref(5, 11) = 73.01023                   'dBV -  2  mV/div

CLS
CALL SendIFC(0)
CALL IBTMO(0, 13)                           'Time out of 10 seconds
CALL Send(0, 8, "*CLS", 1)                  'Clears status data
PRINT "Make the following selections first:"
PRINT "> MATH-MATH1 'on' and 'fft'."
PRINT "> CURSORS 'on' and 'm1.1'."
PRINT "> MATH-PARAM-READOUT select 'abs' or 'rel'."
PRINT "Notes: - CURSORS-READOUT allows to select absolute units."
PRINT "       - the TRACK-control button allows to move the cursor."
PRINT "Press any key to continue."
WHILE INKEY$ = "": WEND

CALL Send(0, 8, "TRACe:POINts? CH1", 1)     'Queries acquisition length
CALL Receive(0, 8, acqlength$, 256)         'Reads acquisition length
acq.length = VAL(LEFT$(acqlength$, IBCNT%))
PRINT "Acq. length="; acq.length; "points"

CALL Send(0, 8, "SENSe:SWEep:TIME?", 1)          'Queries sweep time
CALL Receive(0, 8, sweeptime$, 256)         'Reads sweep time
sweep.time = VAL(LEFT$(sweeptime$, IBCNT%))
PRINT "Sweep time ="; sweep.time; "seconds"

mtb# = sweep.time * 50 / (acq.length - 1)
PRINT "MTB        ="; mtb#
calc# = (((1250 / acq.length) / mtb#) / 50)
'PRINT "calc factor="; calc#

CALL Send(0, 8, "SENSe:VOLtage:RANGe:PTPeak?", 1)  'Queries peak-to-peak
CALL Receive(0, 8, peak.to.peak$, 256)             'Reads peak-to-peak
att = VAL(LEFT$(peak.to.peak$, IBCNT%)) / 8
PRINT "Attenuation ="; att; "V/div."
CALL errorcheck
FOR index = 1 TO 11
   IF ref(1, index) = att THEN              'Determines table index
      EXIT FOR
   END IF
NEXT index
'PRINT "tab. ref. ="; index

CALL Send(0, 8, "CALCulate:TRANsform:FREQuency:TYPE?", 1)
type$ = "    "
CALL Receive(0, 8, type$, 256)                     'Reads FFT-type
PRINT "FFT - type  = "; LEFT$(type$, IBCNT% - 1)

CALL Send(0, 8, "TRACe? M1_1", 1)          'Queries for the M1_1 trace
CALL Receive(0, 8, response$, 256)         'Reads the M1_1 trace
PRINT "Number of read bytes   ="; IBCNT%   'IBCNT% = length of trace buffer

nr.of.digits = VAL(MID$(response$, 2, 1))
nr.of.bytes = VAL(MID$(response$, 3, nr.of.digits)) - 2
sample.length = ASC(MID$(response$, 3 + nr.of.digits, 1))
nr.of.samples = nr.of.bytes / (sample.length / 8)
PRINT "Number of trace samples="; nr.of.samples
IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
   PRINT "Nr.    Frequency   dBm-50  dBm-600    dBV       Vrms"
END IF

IF sample.length = 8 THEN
   FOR i = 1 TO nr.of.samples                 '1-byte samples
      trace&(i) = ASC(MID$(response$, i + 3 + nr.of.digits, 1))
      IF trace&(i) > 127 THEN
	 trace&(i) = trace&(i) - 256
      END IF
      freq(i) = (i - 1) * calc#
      amdb(i) = (trace&(i) - 100) * .4           '0.4 = -10 (dB) / -25
      IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
	 amdbm50(i) = amdb(i) + ref(3, index)    'dBm - 50
	 amdbm600(i) = amdb(i) + ref(4, index)   'dBm - 600
	 amdbmuv(i) = amdb(i) + ref(5, index)    'dBV
	 temp = amdb(i) / 20
	 amvrms(i) = (10 ^ temp) * ref(2, index) 'Vrms
      END IF
      IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
	 PRINT USING "###"; i;
	 'PRINT trace&(i);
	 PRINT USING "#########.###  "; freq(i);
	 PRINT USING "###.####  "; amdbm50(i);
	 PRINT USING "###.####  "; amdbm600(i);
	 PRINT USING "###.####  "; amdbmuv(i);
	 PRINT USING "###.######"; amvrms(i)
      ELSE                                      'RELative FFT
	 PRINT USING "###  "; i;
	 PRINT "Frequency ="; freq(i); "Hz",
	 PRINT " Amplitude ="; amdb(i); "dB"
      END IF
      IF i MOD 20 = 13 THEN
	 PRINT "Press any key for next printout."
	 IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
	    PRINT "Nr.    Frequency   dBm-50  dBm-600    dBV       Vrms"
	 END IF
	 WHILE INKEY$ = "": WEND
      END IF
   NEXT i
ELSE
   FOR i = 1 TO nr.of.samples                   '2-byte samples
      J = 2 * i + 2 + nr.of.digits              'Pointer to next sample
      byte1 = ASC(MID$(response$, J, 1))        'M.S.B.
      byte2 = ASC(MID$(response$, J + 1, 1))    'L.S.B.
      IF byte1 < 128 THEN
	 trace&(i) = byte1 * 256 + byte2
	 ELSE trace&(i) = (byte1 - 256) * 256 + byte2
      END IF
      freq(i) = (i - 1) * calc#
      amdb(i) = (trace&(i) - 25600) * .0015625        '.0015625 = .4 / 256
      IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
	 amdbm50(i) = amdb(i) + ref(3, index)    'dBm - 50
	 amdbm600(i) = amdb(i) + ref(4, index)   'dBm - 600
	 amdbmuv(i) = amdb(i) + ref(5, index)    'dBV
	 temp = amdb(i) / 20
	 amvrms(i) = (10 ^ temp) * ref(2, index) 'Vrms
      END IF
      IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
	 PRINT USING "###"; i;
	 'PRINT trace&(i);
	 PRINT USING "#########.###  "; freq(i);
	 PRINT USING "###.####  "; amdbm50(i);
	 PRINT USING "###.####  "; amdbm600(i);
	 PRINT USING "###.####  "; amdbmuv(i);
	 PRINT USING "###.######"; amvrms(i)
      ELSE                                      'RELative FFT
	 PRINT USING "###  "; i;
	 PRINT "Frequency ="; freq(i); "Hz",
	 PRINT " Amplitude ="; amdb(i); "dB"
      END IF
      IF i MOD 20 = 13 THEN
	 PRINT "Press any key for next printout."
	 IF LEFT$(type$, 1) = "A" THEN              'ABSolute FFT
	    PRINT "Nr.    Frequency   dBm-50  dBm-600    dBV       Vrms"
	 END IF
	 WHILE INKEY$ = "": WEND
      END IF
   NEXT i
END IF

CALL errorcheck
END

'
' **************************************************
' Subroutine reading one error from the error queue.
' **************************************************
SUB errorcheck
  er$ = SPACE$(1)
  WHILE LEFT$(er$, 1) <> "0"
  CMD$ = ":SYSTem:ERRor?"
  CALL Send(0, 8, CMD$, 1)            'Sends error query
  er$ = SPACE$(60)
  CALL Receive(0, 8, er$, 256)        'Reads error string
  PRINT "error = "; er$               'Displays error string
  WEND
END SUB

