REM $INCLUDE: 'QBDECL.BAS'
'*****              *****
'***** EXGETSTA.BAS *****
'***** EXamples GETting STArted in the SCPI Programming Manual.
'*****              *****
DECLARE SUB errorcheck ()
CLS
CALL SendIFC(0)
CALL IbTMO(0, 13)              'Time out of 10 seconds
CALL Send(0, 8, "*RST", 1)     'Resets the instrument
CALL Send(0, 8, "*CLS", 1)     'Clears the status data

PRINT "Continue with identifying your CombiScope instrument?"
PRINT ">>> Press any key to continue or Ctrl/Break to stop."
WHILE INKEY$ = "": WEND

response$ = SPACE$(75)
CALL Send(0, 8, "*IDN?", 1)                     'Requests identification
CALL Receive(0, 8, response$, 256)              'Reads the ident string
PRINT "Ident  : "; LEFT$(response$, IBCNT% - 1) 'Prints the ident string
CALL Send(0, 8, "*OPT?", 1)                     'Requests options
CALL Receive(0, 8, response$, 256)              'Reads the options string
PRINT "Options: "; LEFT$(response$, IBCNT% - 1) 'Prints the options string
CALL errorcheck

PRINT "Continue with switching from digital to analog and vice versa?"
PRINT ">>> Press any key to continue or Ctrl/Break to stop."
WHILE INKEY$ = "": WEND

CALL Send(0, 8, "INSTrument ANALog", 1)      'Switches to analog mode
CALL Send(0, 8, "INSTrument:NSELect 1", 1)   'Switches back to digital mode
CALL errorcheck

PRINT "Continue with acquiring a single shot trace from channel 1?"
PRINT ">>> Connect a probe to channel 1."
PRINT ">>> Press any key to continue or Ctrl/Break to stop."
WHILE INKEY$ = "": WEND

DIM tracebuf AS STRING * 16500
CALL Send(0, 8, "FORMat INTeger,8", 1)         'Formats 8-bits sample
CALL Send(0, 8, "TRACe:POINts CH1,8192", 1)    'Formats 8192 sample points
CALL Send(0, 8, "TRIGger:SOURce INTernal1", 1) 'Trigger-source = channel 1
CALL Send(0, 8, "TRIGger:LEVel 0.1", 1)        'Trigger-level = 0.1
CALL Send(0, 8, "INITiate", 1)                 'Single shot initiation
PRINT ">>>"
PRINT ">>> Trigger your CombiScope by touching the probe tip."
PRINT ">>> Press any key when finished."
WHILE INKEY$ = "": WEND
CALL Send(0, 8, "*WAI", 1)           'Waits for previous commands to finish
CALL Send(0, 8, "TRACe? CH1", 1)     'Queries channel 1 trace
CALL Receive(0, 8, tracebuf$, 256)   'Reads channel 1 trace
'
'The contents of the tracebuf$ string is as follows:
'# 4 8194 <8> <byte 1> ... <byte 8192> <sum> <10>
'
nr.of.digits = VAL(MID$(tracebuf$, 2, 1))                                                         '4 digits
nr.of.bytes = VAL(MID$(tracebuf$, 3, nr.of.digits)) - 2
sample.length = ASC(MID$(tracebuf$, 3 + nr.of.digits, 1)) / 8                                         'sample = 1 (8/8) or 2 (16/8)
nr.of.samples = nr.of.bytes / sample.length
PRINT "Number of bytes received ="; IBCNT%    'IBCNT% = number of bytes
PRINT "Number of trace samples  ="; nr.of.samples
CALL errorcheck

PRINT "Continue with acquiring 5 sequential traces from channel 2?"
PRINT ">>> Connect a probe to channel 2."
PRINT ">>> Press any key to continue or Ctrl/Break to stop."
WHILE INKEY$ = "": WEND

CALL Send(0, 8, "*RST", 1)                    'Resets the instrument
'
'After *RST a trace acquisition is defined at 512 samples of 16 bits
'(2 bytes).
'
CALL Send(0, 8, "CONFigure:AC (@2)", 1)               'Configures channel 2
CALL Send(0, 8, "SENSe:FUNCtion 'XTIMe:VOLTage2'", 1) 'Switches channel 2 on
OPEN "O", #1, "TRACE5.DAT"                            'Opens file TRACE5.DAT
  FOR i = 1 TO 5
    CALL Send(0, 8, "INITiate", 1)                    'Single shot initiation
    CALL Send(0, 8, "*WAI;TRACe? CH2", 1)             'Queries channel 2 trace
    '
    'Notice the *WAI; before TRACe?. The *WAI command takes care that the
    'TRACe? CH2 command is executed when the INITiate command is finished.
    '
    CALL Receive(0, 8, tracebuf$, 256)                'Reads channel 2 trace
    '
    'The contents of the tracebuf$ string is as follows:
    '# 4 1026 <16> <byte 1> ... <byte 1024> <sum> <10>
    '
    PRINT i; ": Number of bytes received ="; IBCNT%   'IBCNT% = number of bytes
    PRINT #1, "Trace buffer:"; i                      'Writes trace header to file
    PRINT #1, LEFT$(tracebuf$, IBCNT%)                'Writes trace buffer to file
  NEXT i
CLOSE                                                 'Closes file TRACE5.DAT
CALL errorcheck

PRINT "Continue with making a single shot measurement from channel 1?"
PRINT ">>> Connect a probe to channel 1."
PRINT ">>> Press any key to continue or Ctrl/Break to stop."
WHILE INKEY$ = "": WEND

response$ = SPACE$(30)
CALL Send(0, 8, "MEASure:AC? (@1)", 1)          'Measures AC-RMS value
CALL Receive(0, 8, response$, 256)              'Reads AC-RMS value
PRINT "AC-RMS value      : "; LEFT$(response$, IBCNT% - 1)
CALL Send(0, 8, "FETCh:PTPeak?", 1)             'Fetches Peak-To-Peak value
CALL Receive(0, 8, response$, 256)              'Reads Peak-To-Peak value
PRINT "Peak-To-Peak value: "; LEFT$(response$, IBCNT% - 1)
CALL Send(0, 8, "FETCh:AMPLitude?", 1)          'Fetches amplitude value
CALL Receive(0, 8, response$, 256)              'Reads amplitude value
PRINT "Amplitude value   : "; LEFT$(response$, IBCNT% - 1)
CALL errorcheck

PRINT "Continue with making repeated measurements from channel 1?"
PRINT ">>> Press any key to continue or Ctrl/Break to stop."
WHILE INKEY$ = "": WEND

response$ = SPACE$(30)
CALL Send(0, 8, "CONFigure:AC (@1)", 1)         'Configures for AC-RMS
FOR i = 1 TO 5                                  'Performs 5 measurements
  CALL Send(0, 8, "READ:AC?", 1)                'Initiates AC-RMS reading
  CALL Receive(0, 8, response$, 256)            'Reads AC-RMS value
  PRINT i; ": AC-RMS: "; LEFT$(response$, IBCNT% - 1);
  CALL Send(0, 8, "FETCh:PTPeak?", 1)           'Fetches Peak-To-Peak value
  CALL Receive(0, 8, response$, 256)            'Reads Peak-To-Peak value
  PRINT " / Peak-To-Peak: "; LEFT$(response$, IBCNT% - 1);
  CALL Send(0, 8, "FETCh:AMPLitude?", 1)        'Fetches amplitude value
  CALL Receive(0, 8, response$, 256)            'Reads amplitude value
  PRINT " / Amplitude: "; LEFT$(response$, IBCNT% - 1)
NEXT i
CALL errorcheck

END

'
' **************************************************
' Subroutine reading one error from the error queue.
' **************************************************
SUB errorcheck
  er$ = SPACE$(1)
  WHILE LEFT$(er$, 1) <> "0"
  CMD$ = "SYSTem:ERRor?"
  CALL Send(0, 8, CMD$, 1)            'Sends error query
  er$ = SPACE$(60)
  CALL Receive(0, 8, er$, 256)        'Reads error string
  PRINT "error = "; er$               'Displays error string
  WEND
END SUB

